package com.cashctrl.stripeimport;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;

import java.io.Closeable;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

/**
 * Parses a stripe csv file.
 * @author Silian Barlogis
 */
public class StripeParser {
    private static final String[] COLUMN_AMOUNT = {"converted_amount", "Converted Amount"};
    private static final String[] COLUMN_DESCRIPTION = {"description", "Description"};
    private static final String[] COLUMN_FEE = {"fee", "Fee"};
    private static final String[] COLUMN_CREATED = {"created", "Created", "Created (UTC)"};
    private static final String[] COLUMN_STATUS = {"status", "Status"};
    private static final String[] COLUMN_CURRENCY = {"converted currency", "Converted Currency"};
    private String[] METADATA_COLUMN_INVOICE = new String[]{""};
    private String[] METADATA_COLUMN_CUSTOMER = new String[]{""};

    private static final int REQUIRED_COLUMN_COUNT = 5;
    private static final CSVFormat format = CSVFormat.DEFAULT.withFirstRecordAsHeader().withIgnoreHeaderCase();

    private final String revenueId;
    private final String expenseId;
    private final String balanceId;
    private Closeable input;

    /**
     * Set the metadata fields.
     * @param invoiceField invoice field
     * @param customerField customer field
     */
    public void setMetadataFields(String invoiceField, String customerField) {
        if (invoiceField != null && !invoiceField.isEmpty()) {
            this.METADATA_COLUMN_INVOICE = new String[]{invoiceField + " (metadata)", invoiceField};
        }

        if (customerField != null && !customerField.isEmpty()) {
            this.METADATA_COLUMN_CUSTOMER = new String[]{customerField + " (metadata)", customerField};
        }
    }

    private CSVParser csvParser = null;

    /**
     * Constructs a stripe parser
     * @param revenueId revenue account id in CashCtrl
     * @param expenseId expense account id in CashCtrl
     * @param balanceId balance account id in CashCtrl
     * @param input the stripe input
     * @param <T> FileReader or InputStream
     */
    public <T extends Closeable> StripeParser(String revenueId, String expenseId, String balanceId, T input) {
        this.revenueId = revenueId;
        this.expenseId = expenseId;
        this.balanceId = balanceId;

        try {
            if (input instanceof FileReader) {
                csvParser = CSVParser.parse((FileReader) input, format);
                this.input = input;
            } else if (input instanceof InputStream) {
                csvParser = CSVParser.parse((InputStream) input, Charset.defaultCharset(), format);
                this.input = input;
            } else {
                throw new IllegalArgumentException("Input must be of type import java.io.FileReader; or import java.io.InputStream;");
            }

        } catch (IOException e) {
            System.out.println("error: could read stream, message: " + e);
        }
    }

    /**
     * parses the csv stripe payments
     * @return return a list of each entry
     */
    public List<Entry> parse() {
        List<Entry> entries = new ArrayList<>();
        for (CSVRecord record : csvParser) {
            String status = getValue(record, COLUMN_STATUS);
            if (status == null || !status.equalsIgnoreCase("Paid")) {
                continue;
            }

            Entry entry = new Entry(revenueId, expenseId, balanceId,
                    getValue(record, COLUMN_CREATED), getValue(record, COLUMN_DESCRIPTION), getValue(record, COLUMN_AMOUNT),
                    getValue(record, COLUMN_FEE), getValue(record, COLUMN_CURRENCY), getValue(record, METADATA_COLUMN_CUSTOMER),
                    getValue(record, METADATA_COLUMN_INVOICE));

            entries.add(entry);
        }

        return entries;
    }

    /**
     * validate stripe headers
     * @return true if valid
     */
    public boolean isValid() {

        int count = 0;
        for (String headerName : csvParser.getHeaderNames()) {
            if (contains(COLUMN_AMOUNT, headerName) || contains(COLUMN_FEE, headerName)
                    || contains(COLUMN_CREATED, headerName) || contains(COLUMN_STATUS, headerName)) {
                count++;
            }
        }
        return count >= REQUIRED_COLUMN_COUNT;
    }

    /**
     * Get record value as string
     * @param record the record to get the value from
     * @param keys the header to get
     * @return value
     */
    private String getValue(CSVRecord record, String[] keys) {
        for (String key : keys) {
            if (record.isMapped(key)) {
                return record.get(key);
            }
        }
        return null;
    }

    /**
     * Check if content contains any of searchArray values
     * @param searchArray the strings to with
     * @param content the content to search in
     * @return true if one of the strings in search arrays was found inside content
     */
    private boolean contains(String[] searchArray, String content) {
        for (String search : searchArray) {
            if (content.contains(search)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Close all handles
     */
    public void close() {
        try {
            if (csvParser != null && !csvParser.isClosed()) {
                csvParser.close();
            }

            if (input != null) {
                input.close();
            }
        } catch (IOException e) {
            System.out.println("error: failed to close file handle");
        }
    }
}