package com.cashctrl.stripeimport;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import java.io.Closeable;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.List;

/**
 * Starting-point of stripe import.
 * @author Silian Barlogis
 */
public final class StripeImport {

    private static final CLI cli = new CLI();
    private static final Error error = Error.getInstance();
    private static Organization organization;

    /**
     * Program main function
     * @param args commandline arguments
     */
    public static void main(String[] args) {
        if (!cli.parseArguments(args)) {
            return;
        }

        if (cli.wannaSeeHelp()) {
            cli.printHelp();
            return;
        }

        if (!cli.validateArguments()) {
            return;
        }

        FileReader file;
        try {
            file = new FileReader(cli.get("file"));
        } catch (FileNotFoundException ignored) {
            System.out.println("error: file not found : " + cli.get("file"));
            return;
        }

        error.setMode(Error.Mode.Full);
        error.setLanguage(Error.Language.English);
        System.out.println("------------ Start import ------------");

        if (importFromStripe(cli.get("org"), cli.get("key"), cli.get("invoiceField"),
                cli.get("customerField"), cli.get("revenue"), cli.get("expense"),
                cli.get("balance"), file)) {
            System.out.println("Successfully imported " + cli.get("file") + " into " + cli.get("org"));
        } else {
            System.out.println("Failed to import " + cli.get("file") + " into " + cli.get("org"));
        }

        System.out.println(error.get());
        error.clear();
    }

    /**
     * Imports a stripe file or input stream into CashCtrl
     * @param revenue the revenue account number
     * @param expense the expense account number
     * @param balance the balance account number
     * @param input stripe file-reader or input stream
     * @param <T> Input-stream or file-reader
     * @return true if successfully imported stripe payments into CashCtrl
     */
    public static <T extends Closeable> boolean importFromStripe(String orgName, String key,
                                                                 String invoiceField, String customerField,
                                                                 String revenue, String expense,
                                                                 String balance, T input) {
        organization = new Organization(orgName, key);
        Journal journal = new Journal();
        StringBuilder revenueId = new StringBuilder(), expenseId = new StringBuilder(), balanceId = new StringBuilder();
        if (!getAccountIds(revenue, expense, balance, revenueId, expenseId, balanceId,
                journal.list(organization, "/api/v1/account/list.json"))) {
            return false;
        }

        StripeParser stripe = new StripeParser(revenueId.toString(), expenseId.toString(), balanceId.toString(), input);
        if (!stripe.isValid()) {

            error.appendLine(new Error.Message("Error: Not a valid stripe csv.", "Fehler: Das Stripe CSV ist fehlerhaft."));
            return false;
        }

        stripe.setMetadataFields(invoiceField, customerField);
        List<Entry> entries = stripe.parse();
        if (entries == null || entries.size() == 0) {
            error.appendLine(new Error.Message("Error: No \"Paid\" payments found.", "Fehler: Keine bezahlten (Paid) Zahlungen gefunden."));
            return false;
        }

        entries.forEach((entry -> {
            if (!journal.createEntry(entry, organization)) {
                error.appendLine(new Error.Message("Error: Failed to create " + entry + " inside " + organization.getUrl() + ".",
                        "Fehler: Das erstellen von " + entry + " in " + organization.getUrl() + " ist fehlgeschlagen."));
            }
        }));
        stripe.close();

        return true;
    }

    /**
     * Try to find the debit and credit ids in organization
     * @param revenue revenue account number
     * @param expense expense account number
     * @param balance balance account number
     * @param balanceId out revenue id
     * @param revenueId out expense id
     * @param expenseId out balance id
     * @param accounts all accounts from organization
     * @return true if all account ids where found
     */
    private static boolean getAccountIds(String revenue, String expense, String balance,
                                         StringBuilder revenueId, StringBuilder expenseId,
                                         StringBuilder balanceId, JsonArray accounts) {
        if (accounts == null) {
            error.appendLine(new Error.Message("Error: Failed to get accounts from organization " + organization.getUrl() + ".",
                    "Fehler: Konten von der Organisation " + organization.getUrl() + " können nicht abgerufen werden."));
            return false;
        }

        for (int idx = 0; idx < accounts.size(); idx++) {
            final JsonObject account = accounts.get(idx).getAsJsonObject();
            if (Journal.has(account, "number")) {
                String number = account.get("number").getAsString();

                if (number.equals(revenue)) {
                    revenueId.append(account.get("id").getAsInt());
                }

                if (number.equals(expense)) {
                    expenseId.append(account.get("id").getAsInt());
                }

                if (number.equals(balance)) {
                    balanceId.append(account.get("id").getAsInt());
                }

                if (!revenueId.isEmpty() && !expenseId.isEmpty() && !balanceId.isEmpty()) {
                    return true;
                }
            }
        }

        if (revenueId.isEmpty()) {
            error.appendLine(new Error.Message("Error: Could not find " + revenue + " revenue account.",
                    "Fehler: Konnte das Ertragskonto " + revenue + " nicht finden."));
        }
        if (expenseId.isEmpty()) {
            error.appendLine(new Error.Message("Error: Could not find " + expense + " expense account.",
                    "Fehler: Konnte das Aufwandskonto " + expense + " nicht finden."));
        }
        if (balanceId.isEmpty()) {
            error.appendLine(new Error.Message("Error: Could not find " + balance + " stripe account.",
                    "Fehler: Konnte das Stripekonto " + balance + " nicht finden."));
        }

        return false;
    }
}